// Copyright 2009-2010 Oliver Sturm <oliver@oliversturm.com>. All rights reserved.

using System;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Collections.Generic;
using System.Collections;

namespace FCSlib.Data.Collections {
  public abstract class ComparableContainer<T> : IComparer<T>, IComparer {
    #region Comparer fields and properties
    private readonly Comparer comparer;
    public Comparer Comparer {
      get {
        return comparer;
      }
    }

    private readonly Comparer<T> genericComparer;
    public Comparer<T> GenericComparer {
      get {
        return genericComparer;
      }
    }

    private readonly Comparison<T> genericComparison;
    public Comparison<T> GenericComparison {
      get {
        return genericComparison;
      }
    }
    #endregion

    #region Constructors
    public ComparableContainer( ) {
    }

    public ComparableContainer(Comparer comparer) {
      this.comparer = comparer;
    }

    public ComparableContainer(Comparer<T> genericComparer) {
      this.genericComparer = genericComparer;
    }

    public ComparableContainer(Comparison<T> genericComparison) {
      this.genericComparison = genericComparison;
    }

    public ComparableContainer(Comparer comparer, Comparer<T> genericComparer, Comparison<T> genericComparison) {
      this.comparer = comparer;
      this.genericComparer = genericComparer;
      this.genericComparison = genericComparison;
    }
    #endregion

    #region Compare
    public int Compare(T x, T y) {
      if (genericComparison != null)
        return genericComparison(x, y);
      else if (genericComparer != null)
        return genericComparer.Compare(x, y);
      else if (comparer != null)
        return comparer.Compare(x, y);
      else {
        return Comparer<T>.Default.Compare(x, y);
      }
    }
    #endregion

    #region Interface implementations
    int IComparer.Compare(object x, object y) {
      return this.Compare((T) x, (T) y);
    }

    int IComparer<T>.Compare(T x, T y) {
      return this.Compare(x, y);
    }
    #endregion
  }
}
