 {-# OPTIONS -XMultiParamTypeClasses -XFlexibleInstances #-}

-- From Chris Okasaki's great book "Purely Functional Data Structures"
-- Get it!

class Set s a where
	empty :: s a
	insert :: a -> s a -> s a
	member :: a -> s a -> Bool

data Color = R | B
	deriving (Show)
	
data RedBlackSet a = E 
					| T Color (RedBlackSet a) a (RedBlackSet a)
					deriving (Show)
										
instance Ord a => Set RedBlackSet a where
	empty = E
	
	member x E = False
	member x (T _ a y b) = 
		if x < y then member x a
		else if x > y then member x b 
		else True
		
	insert x s = T B a y b
		where 
			balance B (T R (T R a x b) y c) z d = T R (T B a x b) y (T B c z d)
			balance B (T R a x (T R b y c)) z d = T R (T B a x b) y (T B c z d)
			balance B a x (T R (T R b y c) z d) = T R (T B a x b) y (T B c z d)
			balance B a x (T R b y (T R c z d)) = T R (T B a x b) y (T B c z d)
			balance color a x b = T color a x b 
			ins E = T R E x E
			ins s@(T color a y b)
				| x < y = balance color (ins a) y b
				| x > y = balance color a y (ins b)
				| otherwise = s
			-- The way this mechanism works together with the T B a y b thing above
			-- somehow makes my brain rotate inplace. The point of it is to do drop
			-- the R from the node returned by ins and use B instead. Would have to dig
			-- into the algorithm in more depth to see why this might be important...
			T _ a y b = ins s

fromList :: Ord a => [a] -> RedBlackSet a
fromList [] = empty
fromList (x:xs) = insert x (fromList xs)

