import Control.Parallel (par, pseq)
import Data.Time.Clock (diffUTCTime, getCurrentTime)
import System.Environment (getArgs)
import System.Random (StdGen, getStdGen, randoms)

-- Pretty cool implementation of a quicksort-like sorting,
-- original source unknown to me.
-- 
-- Testing code mostly taken from Real World Haskell
-- due to laziness :-) Great book, get it!

sort :: Ord v => [v] -> [v]
sort (x:xs) = smaller ++ x:greater
	where 
		smaller = sort [y | y <- xs, y < x]
		greater = sort [y | y <- xs, y >= x]

sort _ = []

force :: [v] -> ()
force xs = go xs `pseq` ()
	where
		go (_:xs) = go xs
		go [] = 1

psort :: Ord v => [v] -> [v]
psort (x:xs) = force greater `par` (force smaller `pseq` (smaller ++ x:greater))
	where 
		smaller = psort [y | y <- xs, y < x]
		greater = psort [y | y <- xs, y >= x]

psort _ = []

fasterpsort :: Ord v => Int -> [v] -> [v]
fasterpsort depth list@(x:xs) 
	| depth <= 0 = sort list
	| otherwise = force greater `par` (force smaller `pseq` (smaller ++ x:greater))
		where
			depth' = depth - 1
			smaller = fasterpsort depth' [y | y <- xs, y < x]
			greater = fasterpsort depth' [y | y <- xs, y >= x]

fasterpsort _ _ = []


randomInts :: Int -> StdGen -> [Int]
randomInts k g = let result = take k (randoms g)
					in force result `seq` result

testFunction = sort
-- testFunction = psort
-- testFunction = fasterpsort 2

main = do
	args <- getArgs
	let count | null args = 500000
			  | otherwise = read (head args)
	input <- randomInts count `fmap` getStdGen
	putStrLn $ "There are " ++ show (length input) ++ " elements to sort"
	start <- getCurrentTime
	let sorted = testFunction input
	putStrLn $ "Sorted " ++ show (length sorted) ++ " elements"
	end <- getCurrentTime
	putStrLn $ show (end `diffUTCTime` start) ++ " elapsed"
